package de.fitconnect.client;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.fitconnect.api.auth.OAuthService;
import de.fitconnect.api.auth.OAuthToken;
import de.fitconnect.api.logger.SdkLogger;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.HashMap;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

import static java.util.stream.Collectors.joining;

public class FitCoAuthService implements OAuthService {

    private static final Logger logger = SdkLogger.defaultLogger(FitCoAuthService.class);
    private static final ObjectMapper mapper = new ObjectMapper();

    private final HttpClient client;
    private final String tokenUrl;

    public FitCoAuthService(final HttpClient client, final String tokenUrl) {
        this.client = client;
        this.tokenUrl = tokenUrl;
    }

    @Override
    public Optional<OAuthToken> authenticate(String clientId, String clientSecret, String scope) {
        final String requestBody = buildRequestBody(clientId, clientSecret, scope);
        try {
            return Optional.of(performTokenRequest(requestBody));
        } catch (IOException | InterruptedException e) {
            logger.log(Level.SEVERE, "error retrieving access token", e);
            return Optional.empty();
        }
    }

    private String buildRequestBody(String clientId, String clientSecret, String scope) {
        var data = new HashMap<String, String>() {{
            put("grant_type", "client_credentials");
            put("client_id", clientId);
            put("client_secret", clientSecret);
            put("scope", scope);
        }};

        return data.entrySet()
                .stream()
                .map(e -> e.getKey() + "=" + e.getValue())
                .collect(joining("&"));
    }

    private OAuthToken performTokenRequest(final String requestBody) throws IOException, InterruptedException {
        HttpRequest request = HttpRequest.newBuilder()
                .header("Accept", "application/json")
                .header("Content-Type","application/x-www-form-urlencoded;charset=UTF-8")
                .uri(URI.create(tokenUrl))
                .POST(HttpRequest.BodyPublishers.ofString(requestBody))
                .build();
        var responseBody = client.send(request, HttpResponse.BodyHandlers.ofString()).body();
        return mapper.readValue(responseBody,OAuthToken.class);
    }
}
